#!/usr/bin/perl -w
# 
# The Russian charmaps
#

$CHARMAP = "(koi8-r|cp1251|iso-8859-5|cp866)";

die "Usage: $0 [path_to_charmaps]\n" if $#ARGV != 0;
die "Cannot find path $ARGV[0]\n" if ! -d $ARGV[0];

%CYRILLIC_NAME = (
    0x0430 => 'Cyrillic_a',        0x0410 => 'Cyrillic_A',
    0x0431 => 'Cyrillic_be',       0x0411 => 'Cyrillic_BE',
    0x0432 => 'Cyrillic_ve',       0x0412 => 'Cyrillic_VE',
    0x0433 => 'Cyrillic_ghe',      0x0413 => 'Cyrillic_GHE',
    0x0434 => 'Cyrillic_de',       0x0414 => 'Cyrillic_DE',
    0x0435 => 'Cyrillic_ie',       0x0415 => 'Cyrillic_IE',
    0x0451 => 'Cyrillic_io',       0x0401 => 'Cyrillic_IO',
    0x0436 => 'Cyrillic_zhe',      0x0416 => 'Cyrillic_ZHE',
    0x0437 => 'Cyrillic_ze',       0x0417 => 'Cyrillic_ZE',
    0x0438 => 'Cyrillic_i',        0x0418 => 'Cyrillic_I',
    0x0439 => 'Cyrillic_shorti',   0x0419 => 'Cyrillic_SHORTI',
    0x043A => 'Cyrillic_ka',       0x041A => 'Cyrillic_KA',
    0x043B => 'Cyrillic_el',       0x041B => 'Cyrillic_EL',
    0x043C => 'Cyrillic_em',       0x041C => 'Cyrillic_EM',
    0x043D => 'Cyrillic_en',       0x041D => 'Cyrillic_EN',
    0x043E => 'Cyrillic_o',        0x041E => 'Cyrillic_O',
    0x043F => 'Cyrillic_pe',       0x041F => 'Cyrillic_PE',
    0x0440 => 'Cyrillic_er',       0x0420 => 'Cyrillic_ER',
    0x0441 => 'Cyrillic_es',       0x0421 => 'Cyrillic_ES',
    0x0442 => 'Cyrillic_te',       0x0422 => 'Cyrillic_TE',
    0x0443 => 'Cyrillic_u',        0x0423 => 'Cyrillic_U',
    0x0444 => 'Cyrillic_ef',       0x0424 => 'Cyrillic_EF',
    0x0445 => 'Cyrillic_ha',       0x0425 => 'Cyrillic_HA',
    0x0446 => 'Cyrillic_tse',      0x0426 => 'Cyrillic_TSE',
    0x0447 => 'Cyrillic_che',      0x0427 => 'Cyrillic_CHE',
    0x0448 => 'Cyrillic_sha',      0x0428 => 'Cyrillic_SHA',
    0x0449 => 'Cyrillic_shcha',    0x0429 => 'Cyrillic_SHCHA',
    0x044A => 'Cyrillic_hardsign', 0x042A => 'Cyrillic_HARDSIGN',
    0x044B => 'Cyrillic_yeru',     0x042B => 'Cyrillic_YERU',
    0x044C => 'Cyrillic_softsign', 0x042C => 'Cyrillic_SOFTSIGN',
    0x044D => 'Cyrillic_e',        0x042D => 'Cyrillic_E',
    0x044E => 'Cyrillic_yu',       0x042E => 'Cyrillic_YU',
    0x044F => 'Cyrillic_ya',       0x042F => 'Cyrillic_YA'
);

foreach $code (keys %CYRILLIC_NAME) {
    $charmap{'utf-8'}{$CYRILLIC_NAME{$code}} = sprintf("U+%04X", $code);
}

while (<$ARGV[0]/*>) {
    open(MAP, "<$_") or die "Error: cannot open $_\n";
    undef $map;
    $begin_charmap = 0;
    while (<MAP>) {
        if (/^<code_set_name>\s+(\S+)$/ || /^%\s+alias\s+(\S+)$/) {
            if ($1 =~ /$CHARMAP/i) {
                $map = lc $&;
            }
        } elsif (/^CHARMAP$/) {
            (defined $map ? $begin_charmap = 1 : last);
        } elsif (/^END CHARMAP$/) {
            last;
        } elsif ($begin_charmap && /^\S+\s+\/x(\w+)\s+<U(\w+)>/) {
            if (exists $CYRILLIC_NAME{hex $2}) {
                $charmap{$map}{$CYRILLIC_NAME{hex $2}} = '+' . hex $1;
            }
        }
    }
    close MAP;
}

open(M4, ">charmap.m4");
print M4 "dnl ***\n";
print M4 "dnl ** Choose one of the charmap options for m4:\ndnl **    ";
foreach $map (sort keys %charmap) {
    print M4 " -D$map";
}
print M4 "\ndnl ***\n";
foreach $map (sort keys %charmap) {
    $first = 1;
    print M4 "\nifdef(`$map',\n`";
    foreach $m (sort keys %{$charmap{$map}}) {
        ($first ? $first = 0 : print M4 "\n");
        print M4 "define(`$m', `$charmap{$map}{$m}')";
    }
    print M4 "')\n";
}
close M4;
print "Created charmap.m4\n";
