#!/usr/bin/perl -w

die "Usage: $0 [path_to_xkb_symbols]\n" if $#ARGV != 0;

sub cnv {
    my $d = $_[0];
    my @digit = ('zero', 'one', 'two', 'three', 'four',
                 'five', 'six', 'seven', 'eight', 'nine');
    if ($d =~ /^\d$/) {
        return $digit[$d];
    } elsif ($d =~ /^\w$/) {
        return '+'. $d;
    } else {
        return $d;
    }
}

$/ = ';';
$regexp = '\s*key\s*<(\w+)>\s*{\s*\[\s*(\w+)\s*,\s*(\w+)\s*\]\s*,' .
          '\s*\[\s*(\w+)\s*,\s*(\w+)\s*\]\s*}';

open(XKB, "<$ARGV[0]") or die "Error: cannot open $ARGV[0]\n";
while (<XKB>) {
    if (/xkb_symbols\s+"(\w+)"\s+{/) {
        $lay = $1;
    }
    if (/include\s+"(\w+)\((\w+)\)"/s) {
        die "Error: cannot $&\n" if ($1 ne $ARGV[0]);
        %{$layout{$lay}} = %{$layout{$2}};
    }
    if (/$regexp/) {
        $layout{$lay}{$1} = [cnv($2), cnv($3), cnv($4), cnv($5)];
    }
}
close XKB;

open(M4, ">layout.m4");
print M4 "dnl ***\n";
print M4 "dnl ** Choose one of the layout options for m4:\ndnl **    ";
foreach $lay (sort keys %layout) {
    print M4 " -D$lay";
}
print M4 "\ndnl ***\n";
foreach $lay (sort keys %layout) {
    $first = 1;
    print M4 "\nifdef(`$lay',\n`";
    foreach $k (sort keys %{$layout{$lay}}) {
        ($first ? $first = 0 : print M4 "\n");
        print M4 "define(`Xkb_${k}_1', `$layout{$lay}{$k}[0]')\n";
        print M4 "define(`Xkb_${k}_2', `$layout{$lay}{$k}[1]')\n";
        print M4 "define(`Xkb_${k}_3', `$layout{$lay}{$k}[2]')\n";
        print M4 "define(`Xkb_${k}_4', `$layout{$lay}{$k}[3]')";
    }
    print M4 "')\n";
}
close M4;
print "Created layout.m4\n";
